﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.IO;
using System.Linq;
using CookComputing.XmlRpc;
using ExportToWordpress.XMLRPC;
using MigrateSharePointToWordPress.Controls;
using MigrateSharePointToWordPress.Source;

namespace MigrateSharePointToWordPress.Destination
{
	public class WordPressConnection
	{
		private const string BlogId = "blog_id";
		private readonly Log _log;
		private readonly SharePointConnection _source;
		private readonly BackgroundWorker _worker;
		private string _blogUrl;
		private readonly IWordpressXmlRpc _xmlRpcProxy;
		private readonly CredentialData _credentials;

		public string BlogUrl
		{
			get { return _blogUrl; }
			set { _blogUrl = value != null ? value.TrimEnd('/') : null; }
		}

		public WordPressConnection(CredentialData credentialData, SharePointConnection source, BackgroundWorker worker)
		{
			_credentials = credentialData;
			_source = source;
			_worker = worker;
			BlogUrl = credentialData.Url;
			_log = new Log(worker);

			_xmlRpcProxy = (IWordpressXmlRpc)XmlRpcProxyGen.Create(typeof(IWordpressXmlRpc));
			var client = (XmlRpcClientProtocol)_xmlRpcProxy;
			client.Url = BlogUrl + "/xmlrpc.php";
		}

		public void UploadResources(List<BlogPost> blogPosts)
		{
			foreach (BlogPost blogPost in blogPosts)
			{
				foreach (LocalResource localResource in blogPost.LocalResources)
				{
					bool alreadyUploaded = false;

					// check if the file has already been uploaded
					foreach (var post in blogPosts)
					{
						if (alreadyUploaded)
							break;

						if (post.LocalResources.Any(resource => resource.SourceUrl == localResource.SourceUrl && resource.AlreadyUploaded))
						{
							alreadyUploaded = true;
						}
					}

					if (alreadyUploaded)
						continue;

					UploadResource(_xmlRpcProxy, localResource);
				}
			}
		}

		// Upload image
		public void UploadResource(IWordpressXmlRpc client, LocalResource localResource)
		{
			if (localResource.SourcePostId != 0)
			{
				// do not upload a referenced post. need to replace the url after all posts have been created
				return;
			}

			var resource = new WordpressFile
			{
				name = Path.GetFileName(localResource.SourceUrl),
				bits = _source.DownloadFile(localResource.SourceUrl),
				overwrite = false
			};
			resource.type = resource.SetMimeType(resource);
			try
			{
				WordpressFile uploadedImage = client.newImage(BlogId, _credentials.Username, _credentials.Password, resource, false);
				localResource.DestinationUrl = uploadedImage.url;
				localResource.AlreadyUploaded = true;

				_log.Verbose("Uploaded to: " + uploadedImage.url);
			}
			catch (XmlRpcFaultException ex)
			{
				if (ex.HResult == -2146232832)
				{
					_log.Error("Cannot upload the file '" + localResource.OriginalSourceUrl + "'." + " Allowing all filetypes with 'define('ALLOW_UNFILTERED_UPLOADS', true);' might help, but is a potential security risk!", ex);
				}
				else
				{
					_log.Error("Error: " + ex.Message, ex);
					throw;
				}
			}
		}

		public void MigrateData(List<BlogPost> blogPosts)
		{
			_worker.ReportProgress(new ProgressReport(ProgressReport.Indeterminate.Deactivate));
			CreateCategories(blogPosts);

			// get all items to prevent multiple upload later
			MediaItem[] mediaItems = _xmlRpcProxy.getMediaLibrary(BlogId, _credentials.Username, _credentials.Password, new MediaFilter { offset = 0, number = 10000, parent_id = string.Empty });

			_worker.ReportProgress(new ProgressReport("Creating posts in Wordpress...") { Status = "Creating " + blogPosts.Count + " posts..." });
			for (int i = 0; i < blogPosts.Count; i++)
			{
				_worker.ReportProgress(new ProgressReport(i, blogPosts.Count));
				CreatePost(blogPosts[i], mediaItems);
			}

			_log.Info("Updating local post references...");
			List<BlogPost> postsWithLocalReferences = blogPosts.Where(p => p.LocalResources.Any(r => r.SourcePostId != 0)).ToList();
			for (int i = 0; i < postsWithLocalReferences.Count; i++)
			{
				_worker.ReportProgress(new ProgressReport(i, postsWithLocalReferences.Count));
				ModifyLocalPostReferences(blogPosts, blogPosts[i]);
			}
			_log.Info("Migration complete");
		}

		private void CreatePost(BlogPost blogPost, MediaItem[] mediaItems)
		{
			foreach (LocalResource localResource in blogPost.LocalResources.Where(r => !r.AlreadyUploaded))
			{
				string fileName = Path.GetFileName(localResource.OriginalSourceUrl);
				MediaItem item = mediaItems.SingleOrDefault(i => Path.GetFileName(i.link) == fileName);
				if (!string.IsNullOrEmpty(item.link))
				{
					_log.Verbose(fileName + " already exists at the Destination.");
					localResource.DestinationUrl = item.link;
					localResource.AlreadyUploaded = true;
				}
				else
				{
					UploadResource(_xmlRpcProxy, localResource);
				}
				// modify Body to use the uploaded resource
				blogPost.Body = blogPost.Body.Replace(localResource.OriginalSourceUrl, localResource.DestinationUrl);
			}

			NewPostContent post = blogPost.GetPostContent();
			blogPost.DestinationId = int.Parse(_xmlRpcProxy.newPost(BlogId, _credentials.Username, _credentials.Password, post, true));

			AddComments(blogPost);
		}

		private void AddComments(BlogPost blogPost)
		{
			foreach (BlogComment blogComment in blogPost.Comments)
			{
				try
				{
					var comment = new Comment(true)
					{
						comment_parent = 0,
						content = blogComment.Body,
						author = blogComment.Title,
						author_url = blogComment.CommentUrl,
						author_email = blogComment.EMail
					};
					// empty username for anonymous comment
					blogComment.DestinationId = _xmlRpcProxy.newComment(BlogId, _credentials.Username, _credentials.Password, blogPost.DestinationId, comment);
					// some properties can only be set by updating the comment
					comment.date_created_gmt = blogComment.Created;
					_xmlRpcProxy.editComment(BlogId, _credentials.Username, _credentials.Password, blogComment.DestinationId, comment);
				}
				catch (XmlRpcFaultException ex)
				{
					if (ex.HResult == -2146232832)
					{
						_log.Verbose("Duplicate comment on post '" + blogPost.Title);
						continue;
					}
					throw;
				}
			}
		}

		private void CreateCategories(List<BlogPost> blogPosts)
		{
			_log.Info("Creating categories...");
			var categories = new List<string>();
			foreach (BlogPost blogPost in blogPosts)
			{
				foreach (string category in blogPost.Tags.Where(category => !categories.Contains(category)))
				{
					categories.Add(category);
				}
			}
			for (int i = 0; i < categories.Count; i++)
			{
				_worker.ReportProgress(new ProgressReport(i, categories.Count));
				var category = new TaxonomyContent(true) { name = categories[i], slug = categories[i].ToSlug() };
				try
				{
					_xmlRpcProxy.newTerm("1", _credentials.Username, _credentials.Password, category);
				}
				catch (XmlRpcFaultException ex)
				{
					if (ex.HResult == -2146232832)
					{
						// the term already exists
					}
					else
					{
						throw;
					}
				}
			}
		}

		private void ModifyLocalPostReferences(List<BlogPost> blogPosts, BlogPost blogPost)
		{
			foreach (LocalResource resource in blogPost.LocalResources)
			{
				if (resource.SourcePostId == 0)
					continue;

				BlogPost referencedPost = blogPosts.SingleOrDefault(p => p.DestinationId == resource.SourcePostId);
				if (referencedPost == null)
				{
					_log.Error("A post with the old ID " + resource.SourcePostId + " could not be found", null);
				}
				else
				{
					// update link to the new post URL
					blogPost.Body = blogPost.Body.Replace(resource.OriginalSourceUrl, BlogUrl + "/?p=" + referencedPost.DestinationId);
					NewPostContent content = blogPost.GetPostContent();
					_xmlRpcProxy.editPost(BlogId, _credentials.Username, _credentials.Password, content, true);
				}
			}
		}

		/// <summary>
		/// Delete Content
		/// </summary>
		/// <param name="cleanupCount"></param>
		internal void Cleanup(int cleanupCount)
		{
			Comment[] comments = _xmlRpcProxy.getComments(BlogId, _credentials.Username, _credentials.Password, new CommentFilter { offset = 0, number = cleanupCount });
			_worker.ReportProgress(new ProgressReport("Deleting all comments...") { Status = "Deleting " + comments.Length + " comments..." });
			for (int i = 0; i < comments.Length; i++)
			{
				_worker.ReportProgress(new ProgressReport(i, comments.Length));
				int commentId = Convert.ToInt32(comments[i].comment_id);
				_log.Verbose("Deleting Comment with ID " + commentId);
				// move to trash
				_xmlRpcProxy.deleteComment(BlogId, _credentials.Username, _credentials.Password, commentId);
				// delete again to fully delete the post
				_xmlRpcProxy.deleteComment(BlogId, _credentials.Username, _credentials.Password, commentId);
			}

			ExistingPostContent[] posts = _xmlRpcProxy.getRecentPosts(BlogId, _credentials.Username, _credentials.Password, cleanupCount);
			_worker.ReportProgress(new ProgressReport("Deleting all posts...") { Status = "Deleting " + posts.Length + " posts..." });
			for (int i = 0; i < posts.Length; i++)
			{
				_worker.ReportProgress(new ProgressReport(i, posts.Length));
				int postid = Convert.ToInt32(posts[i].postid);
				_log.Verbose("Deleting post with ID " + postid);
				// move to trash
				_xmlRpcProxy.deletePost(BlogId, _credentials.Username, _credentials.Password, postid);
				// delete again to fully delete the post
				_xmlRpcProxy.deletePost(BlogId, _credentials.Username, _credentials.Password, postid);
			}

			MediaItem[] mediaItems = _xmlRpcProxy.getMediaLibrary(BlogId, _credentials.Username, _credentials.Password, new MediaFilter { offset = 0, number = cleanupCount, parent_id = string.Empty });
			_worker.ReportProgress(new ProgressReport("Deleting all media items...") { Status = "Deleting " + mediaItems.Length + " media items..." });
			for (int i = 0; i < mediaItems.Length; i++)
			{
				_worker.ReportProgress(new ProgressReport(i, mediaItems.Length));
				int itemId = Convert.ToInt32(mediaItems[i].attachment_id);
				_log.Verbose("Deleting media item with ID " + itemId);
				// move to trash
				_xmlRpcProxy.deletePost(BlogId, _credentials.Username, _credentials.Password, itemId);
				// delete again to fully delete the post
				//_xmlRpcProxy.deletePage(BlogId, _credentials.Username, _credentials.Password, itemId);
			}

			DeleteTerms(cleanupCount, "category");
			DeleteTerms(cleanupCount, "post_tag");
			_worker.ReportProgress(new ProgressReport(ProgressReport.Indeterminate.Deactivate) { Output = "Done cleaning up." });
		}

		private void DeleteTerms(int cleanupCount, string taxonomyType)
		{
			Term[] categories = _xmlRpcProxy.getTerms(BlogId, _credentials.Username, _credentials.Password, taxonomyType, new TermFilter { hide_empty = false, offset = 0, number = cleanupCount });
			_worker.ReportProgress(new ProgressReport("Deleting all " + taxonomyType + "...") { Status = "Deleting " + categories.Length + " " + taxonomyType });

			for (int i = 0; i < categories.Length; i++)
			{
				_worker.ReportProgress(new ProgressReport(i, categories.Length));
				Term term = categories[i];
				var termId = term.term_id;
				if (taxonomyType == "category" && termId == "1")
				{
					_log.Verbose("Skipping category 'Uncategorized'");
					continue;
				}
				_log.Verbose("Deleting media item with name '" + term.name + "' and term_id " + termId);
				_xmlRpcProxy.deleteTerm(BlogId, _credentials.Username, _credentials.Password, taxonomyType, Convert.ToInt32(termId));
			}
		}
	}
}