﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Windows;
using MigrateSharePointToWordPress.Controls;
using MigrateSharePointToWordPress.Destination;
using MigrateSharePointToWordPress.Properties;
using MigrateSharePointToWordPress.Source;

namespace MigrateSharePointToWordPress
{
	/// <summary>
	/// Interaction logic for MainWindow.xaml
	/// </summary>
	public partial class MainWindow : Window
	{
		public MainWindow()
		{
			InitializeComponent();
			Loaded += LoadSettings;
			Closing += SaveSettings;
		}

		private void LoadSettings(object sender, RoutedEventArgs e)
		{
			SourceCredentials.Url = Settings.Default.Source_Url;
			SourceCredentials.Username = Settings.Default.Source_Username;
			SourceCredentials.UseDefaultCredentials = Settings.Default.Source_UseDefaultCredentials;
			DestinationCredentials.Url = Settings.Default.Destination_Url;
			DestinationCredentials.Username = Settings.Default.Destination_Username;
			DestinationCredentials.UseDefaultCredentials = Settings.Default.Destination_UseDefaultCredentials;
		}

		private void SaveSettings(object sender, CancelEventArgs e)
		{
			Settings.Default.Source_Url = SourceCredentials.Url;
			Settings.Default.Source_Username = SourceCredentials.Username;
			Settings.Default.Source_UseDefaultCredentials = SourceCredentials.UseDefaultCredentials;
			Settings.Default.Destination_Url = DestinationCredentials.Url;
			Settings.Default.Destination_Username = DestinationCredentials.Username;
			Settings.Default.Destination_UseDefaultCredentials = DestinationCredentials.UseDefaultCredentials;
			Settings.Default.Save();
		}

		private void StartMigration(object sender, RoutedEventArgs eventArgs)
		{
			var worker = new BackgroundWorker { WorkerReportsProgress = true };
			worker.ProgressChanged += OnWorkerProgressReporting;
			worker.DoWork += delegate(object s, DoWorkEventArgs e)
			{
				try
				{
					var credentialDatas = e.Argument as CredentialData[];

					var source = new SharePointConnection(credentialDatas[0], worker);
					source.Connect();
					List<BlogPost> blogPosts = source.GetData();

					var destination = new WordPressConnection(credentialDatas[1], source, worker);
					destination.MigrateData(blogPosts);
				}
				catch (Exception ex)
				{
					(new Log(worker)).Error(ex.Message, ex);
					worker.ReportProgress(new ProgressReport(ProgressReport.Indeterminate.Deactivate) { Status = "Error" });
				}
			};
			worker.RunWorkerAsync(new[] { SourceCredentials.GetCredentialData(), DestinationCredentials.GetCredentialData() });
		}

		private void DeleteWordPressContent(object sender, RoutedEventArgs eventArgs)
		{
			var messageBoxResult = MessageBox.Show("Are you sure? All content (posts, categories, tags, comment) is going to be deleted!", "Delete Confirmation", MessageBoxButton.YesNo);
			if (messageBoxResult == MessageBoxResult.Yes)
			{
				var worker = new BackgroundWorker { WorkerReportsProgress = true };
				worker.DoWork += delegate(object s, DoWorkEventArgs e)
				{
					try
					{
						var destinationData = e.Argument as CredentialData;
						var destination = new WordPressConnection(destinationData, null, worker);
						destination.Cleanup(1000);
					}
					catch (Exception ex)
					{
						(new Log(worker)).Error(ex.Message, ex);
						worker.ReportProgress(new ProgressReport(ProgressReport.Indeterminate.Deactivate) { Status = "Error" });
					}
				};
				worker.ProgressChanged += OnWorkerProgressReporting;
				worker.RunWorkerAsync(DestinationCredentials.GetCredentialData());
			}
		}

		private void OnWorkerProgressReporting(object sender, ProgressChangedEventArgs progressChangedEventArgs)
		{
			var progress = progressChangedEventArgs.UserState as ProgressReport;
			switch (progress.IndeterminateStatus)
			{
				case ProgressReport.Indeterminate.Activate:
					MainProgressBar.IsIndeterminate = true;
					StatusLabel.Content = null;
					break;
				case ProgressReport.Indeterminate.Deactivate:
					MainProgressBar.IsIndeterminate = false;
					MainProgressBar.Value = 0;
					StatusLabel.Content = null;
					break;
			}

			if (progress.IsPercentage)
			{
				MainProgressBar.Value = progress.Percentage;
				StatusLabel.Content = progress.Status;
			}
			else if (!string.IsNullOrEmpty(progress.Output))
			{
				Output.Text += progress.Output + Environment.NewLine;
				Output.ScrollToEnd();
			}

			if (!string.IsNullOrEmpty(progress.Status))
				StatusLabel.Content = progress.Status;
		}
	}
}
