using System;
using System.Collections.Generic;
using System.Linq;
using System.Text.RegularExpressions;
using System.Xml.Linq;
using ExportToWordpress.XMLRPC;

namespace MigrateSharePointToWordPress.Source
{
	public class BlogPost : IBlogElement
	{
		private readonly Dictionary<string, string> _replaceStrings;
		private string _body;
		public int ID { get; set; }
		public int DestinationId { get; set; }
		public string Title { get; set; }
		public string BlogTitleForUrl { get; set; }
		public DateTime Created { get; set; }
		public DateTime Published { get; set; }
		public string Body
		{
			get { return _body; }
			set { _body = ReplaceStringsInBody(value); }
		}
		public List<string> Tags { get; set; }
		public List<LocalResource> LocalResources { get; set; }
		public List<BlogComment> Comments { get; set; }
		public string EbeUrl { get; set; }

		public BlogPost()
		{
			LocalResources = new List<LocalResource>();
			Comments = new List<BlogComment>();
			_replaceStrings = new Dictionary<string, string>();
		}

		public BlogPost(XElement row, string relativeBlogUrl, Dictionary<string, string> replaceStrings)
			: this()
		{
			_replaceStrings = replaceStrings;
			ID = Convert.ToInt32(row.Attribute("ows_ID").Value);
			Title = row.GetAttributeValue("ows_Title");
			Body = row.GetAttributeValue("ows_Body");

			// get categories from EBE or SP Field
			var tags = row.GetAttributeValue("ows_Categories") ?? row.GetAttributeValue("ows_PostCategory");
			Tags = tags.GetLookupValues();
			Created = Convert.ToDateTime(row.Attribute("ows_Created").Value);
			Published = Convert.ToDateTime(row.Attribute("ows_PublishedDate").Value);
			BlogTitleForUrl = row.GetAttributeValue("ows_BlogTitleForUrl");
			if (!string.IsNullOrEmpty(BlogTitleForUrl))
			{
				// add URL from CKS:EBE
				EbeUrl = string.Format("{0}/archive/{1}/{2}/{3}/{4}.aspx", relativeBlogUrl, Created.Year, Created.ToString("MM"), Created.ToString("dd"), BlogTitleForUrl);
			}
		}

		public NewPostContent GetPostContent()
		{
			var post = new NewPostContent(true)
			{
				title = Title,

				description = Body,
				dateCreated = Published,
				categories = Tags.ToArray(),
				mt_keywords = Tags.ToArray()
			};
			return post;
		}

		/// <summary>
		/// Adds the resource, if the OriginalSourceUrl does not exist
		/// </summary>
		/// <param name="resource"></param>
		public void AddResource(LocalResource resource)
		{
			if (LocalResources.Any(r => r.OriginalSourceUrl == resource.OriginalSourceUrl))
				return;

			// need to check for relative paths by converting absolute url to local path to ignore schema
			string localPath = new Uri(resource.SourceUrl, UriKind.RelativeOrAbsolute).LocalPath;
			LocalResource localResource = LocalResources.SingleOrDefault(r => new Uri(r.SourceUrl, UriKind.RelativeOrAbsolute).LocalPath == localPath);
			if (localResource != null)
			{
				// make URL relative to replace every instance later
				var uri = new Uri(localResource.OriginalSourceUrl, UriKind.RelativeOrAbsolute);
				if (uri.IsAbsoluteUri)
					localResource.OriginalSourceUrl = uri.LocalPath;
				return;
			}

			LocalResources.Add(resource);
		}

		private string ReplaceStringsInBody(string value)
		{
			foreach (KeyValuePair<string, string> replaceString in _replaceStrings)
			{
				string replace = Regex.Replace(value, replaceString.Key, replaceString.Value);
				value = replace;
			}

			return value;
		}
	}
}