﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.IO;
using System.Linq;
using System.Net;
using System.Text;
using System.Text.RegularExpressions;
using System.Xml;
using System.Xml.Linq;
using MigrateSharePointToWordPress.Controls;
using MigrateSharePointToWordPress.ListsService;

namespace MigrateSharePointToWordPress.Source
{
	public class SharePointConnection
	{
		private readonly Log _log;
		private string _blogUrl;
		private Lists _lists;
		private readonly BackgroundWorker _worker;
		private readonly XNamespace _s = "http://schemas.microsoft.com/sharepoint/soap/";
		private readonly XNamespace _rs = "urn:schemas-microsoft-com:rowset";
		private readonly XNamespace _z = "#RowsetSchema";

		/// <summary>
		/// absolute URL to the SharePoint Blog SPWeb
		/// </summary>
		public string BlogUrl
		{
			get { return _blogUrl; }
			set
			{
				_blogUrl = value;
				RelativeBlogUrl = (new Uri(value)).LocalPath;
			}
		}

		/// <summary>
		/// The relative blog URL
		/// </summary>
		internal string RelativeBlogUrl { get; set; }

		/// <summary>
		/// Displayname or GUID
		/// </summary>
		public string PostListName { get; set; }

		private string PostListServerRelativeUrl { get; set; }

		/// <summary>
		/// Displayname or GUID
		/// </summary>
		public string CommentListName { get; set; }

		public SharePointConnection(CredentialData credentialData, BackgroundWorker worker)
		{
			BlogUrl = credentialData.Url;
			_worker = worker;
			_log = new Log(worker);
		}

		/// <summary>
		/// Get posts and comments lists data
		/// </summary>
		public void Connect()
		{
			if (string.IsNullOrEmpty(BlogUrl))
				throw new Exception("The BlogUrl is not set.");

			_lists = new Lists
			{
				Url = string.Format("{0}/_vti_bin/lists.asmx", BlogUrl),
				Credentials = CredentialCache.DefaultNetworkCredentials
			};

			_worker.ReportProgress(new ProgressReport(ProgressReport.Indeterminate.Activate));
			_log.Info("Connecting to '" + BlogUrl + "' to get lists...");
			XDocument response = XDocument.Parse(_lists.GetListCollection().OuterXml);
			IEnumerable<XElement> lists = response.Root.Descendants(XName.Get("List", _s.ToString()));
			foreach (XElement list in lists)
			{
				XAttribute listTemplate = list.Attribute(XName.Get("ServerTemplate"));
				if (listTemplate != null && listTemplate.Value == "301")
				{
					// found Posts list
					PostListName = list.Attribute(XName.Get("Title")).Value;
					PostListServerRelativeUrl = list.Attribute(XName.Get("DefaultViewUrl")).Value.Replace("/AllPosts.aspx", string.Empty);
				}
				else if (listTemplate != null && listTemplate.Value == "302")
				{
					// found Comments list
					CommentListName = list.Attribute(XName.Get("Title")).Value;
				}
			}

			_worker.ReportProgress(new ProgressReport(ProgressReport.Indeterminate.Deactivate));

			if (string.IsNullOrEmpty(PostListName) || string.IsNullOrEmpty(PostListServerRelativeUrl) || string.IsNullOrEmpty(CommentListName))
				throw new InvalidOperationException("Could not find posts and comments list in the specified web.");
		}

		public List<BlogPost> GetData()
		{
			if (_lists == null)
				throw new InvalidOperationException("The connection has to be established first. Call Connect() prior GetData().");

			Dictionary<string, string> replaceStrings = GetReplaceStrings();
			var blogPosts = new List<BlogPost>();
			var postsConfig = new SharePointListConfig(SharePointListConfig.ListType.Posts, PostListName, CommentListName);
			_worker.ReportProgress(new ProgressReport(ProgressReport.Indeterminate.Activate));
			_log.Info("Reading " + postsConfig.ListItemType + "...");
			XDocument response = GetListItems(postsConfig);

			do
			{
				//rs:data ItemCount="30" ListItemCollectionPositionNext="Paged=TRUE&amp;p_ID=33"
				XElement root = response.Root;
				foreach (XElement row in root.Descendants(XName.Get("row", _z.ToString())))
				{
					blogPosts.Add(new BlogPost(row, RelativeBlogUrl, replaceStrings));
				}

				XElement node = root.Descendants(XName.Get("data", _rs.ToString())).First();
				XAttribute nextNode = node.Attribute("ListItemCollectionPositionNext");
				if (nextNode != null)
				{
					postsConfig.ListItemCollectionPosition = nextNode.Value;
					if (!string.IsNullOrEmpty(postsConfig.ListItemCollectionPosition))
					{
						postsConfig.PageSize = node.Attribute("ItemCount").Value;
						_log.Verbose("Reading " + postsConfig.PageSize + " " + postsConfig.ListItemType + "...");
						response = GetListItems(postsConfig);
					}
				}
				else
				{
					postsConfig.PageSize = null;
					postsConfig.ListItemCollectionPosition = null;
				}
			} while (!string.IsNullOrEmpty(postsConfig.PageSize));

			_log.Info("Got " + blogPosts.Count + " posts. Now fetching comments...");
			if (blogPosts.Count > 0)
			{
				IList<BlogComment> comments = GetComments(postsConfig, blogPosts.First());
				foreach (BlogComment comment in comments)
				{
					// add comment to post
					BlogPost post = blogPosts.SingleOrDefault(p => p.ID == comment.PostID);
					if (post != null)
						post.Comments.Add(comment);
				}
				_log.Info("Got " + comments.Count + " comments and added them to their posts.");
			}

			ExtractLocalUrls(blogPosts);
			_worker.ReportProgress(new ProgressReport(ProgressReport.Indeterminate.Deactivate));
			return blogPosts.OrderBy(p => p.ID).ToList();
		}

		private IList<BlogComment> GetComments(SharePointListConfig postsConfig, BlogPost post)
		{
			var comments = new List<BlogComment>();
			SharePointListConfig commentConfig = postsConfig.GetCommentsConfig(post);
			XDocument response = GetListItems(commentConfig);
			do
			{
				//rs:data ItemCount="30" ListItemCollectionPositionNext="Paged=TRUE&amp;p_ID=33"
				XElement root = response.Root;
				foreach (XElement row in root.Descendants(XName.Get("row", _z.ToString())))
				{
					comments.Add(new BlogComment(row));
				}

				XElement node = root.Descendants(XName.Get("data", _rs.ToString())).First();
				XAttribute nextNode = node.Attribute("ListItemCollectionPositionNext");
				if (nextNode != null)
				{
					commentConfig.ListItemCollectionPosition = nextNode.Value;
					if (!string.IsNullOrEmpty(commentConfig.ListItemCollectionPosition))
					{
						commentConfig.PageSize = node.Attribute("ItemCount").Value;
						response = GetListItems(commentConfig);
					}
				}
				else
				{
					commentConfig.PageSize = null;
					commentConfig.ListItemCollectionPosition = null;

				}
			} while (!string.IsNullOrEmpty(commentConfig.PageSize));

			if (post.Comments.Count != 0)
				_log.Info("\tGot " + post.Comments.Count + " comments.");
			return comments;
		}

		private XDocument GetListItems(SharePointListConfig config)
		{
			var xmlDoc = new XmlDocument();

			XmlNode ndQuery = xmlDoc.CreateNode(XmlNodeType.Element, "Query", "");
			XmlNode ndViewFields = xmlDoc.CreateNode(XmlNodeType.Element, "ViewFields", "");
			XmlNode ndQueryOptions = xmlDoc.CreateNode(XmlNodeType.Element, "QueryOptions", "");

			if (!string.IsNullOrEmpty(config.ListItemCollectionPosition))
			{
				ndQueryOptions.InnerXml = string.Format("<IncludeMandatoryColumns>FALSE</IncludeMandatoryColumns><DateInUtc>TRUE</DateInUtc><Paging ListItemCollectionPositionNext=\"{0}\" />",
					config.ListItemCollectionPosition.Replace("&", "&amp;"));
			}
			else
			{
				ndQueryOptions.InnerXml = "<IncludeMandatoryColumns>FALSE</IncludeMandatoryColumns><DateInUtc>TRUE</DateInUtc>";
			}

			// get all comments and posts
			if (config.ListItemType == SharePointListConfig.ListType.Posts)
			{
				// PostCatgory for SP Blog, BlogTitleForUrl and Categories for EBE Blogs
				ndViewFields.InnerXml = "<FieldRef Name='ID' /><FieldRef Name='Title'/><FieldRef Name='Body'/><FieldRef Name='PublishedDate'/><FieldRef Name='BlogTitleForUrl'/><FieldRef Name='Categories'/><FieldRef Name='PostCategory'/><FieldRef Name='Author'/>";
			}
			else
			{
				ndViewFields.InnerXml = "<FieldRef Name='ID' /><FieldRef Name='Title'/><FieldRef Name='Body'/><FieldRef Name='PostTitle'/><FieldRef Name='CommentUrl'/><FieldRef Name='EmailAddress'/><FieldRef Name='Author'/><FieldRef Name='Created'/>";
			}
			try
			{
				XmlNode ndListItems = _lists.GetListItems(config.GetListName(), null, ndQuery, ndViewFields, null, ndQueryOptions, null);
				XDocument response = XDocument.Parse(ndListItems.OuterXml);
				return response;
			}
			catch (System.Web.Services.Protocols.SoapException ex)
			{
				throw new Exception(ex.Message + Environment.NewLine + ex.Detail.InnerText, ex);
			}
		}

		/// <summary>
		/// find references to resources within the blog site
		/// </summary>
		/// <param name="blogPosts"></param>
		public void ExtractLocalUrls(List<BlogPost> blogPosts)
		{
			var blogUri = new Uri(BlogUrl);
			// all posts references 
			var localPostRegex = new Regex("(?<url>" + PostListServerRelativeUrl + @"/Post.aspx\?Id=(?<id>\d+))", RegexOptions.IgnoreCase);
			// all local urls
			//var relativeRegex = new Regex("=\"(?<url>" + blogUri.LocalPath + "/(.)[^\"]+)");
			var relativeRegex = new Regex("=\"(?<url>/(.)[^\"]+)");
			// all absolute urls
			var absoluteRegex = new Regex("=\"(?<url>" + blogUri.AbsoluteUri + "/(.)[^\"]+)");
			Regex absoluteHttpRegex = null;
			if (blogUri.Scheme == "https")
				absoluteHttpRegex = new Regex("=\"(?<url>" + blogUri.AbsoluteUri.Replace("https", "http") + "/(.)[^\"]+)");

			List<BlogPost> ebeUrls = blogPosts.Where(p => !string.IsNullOrEmpty(p.EbeUrl)).ToList();
			// If EBE was installed, check for the URL schema
			if (ebeUrls.Count > 0)
			{
				foreach (BlogPost blogPost in blogPosts)
				{
					// check each post if it references another
					foreach (BlogPost ebeUrl in ebeUrls)
					{
						if (blogPost.Body.Contains(ebeUrl.EbeUrl))
							blogPost.AddResource(new LocalResource(BlogUrl, ebeUrl.EbeUrl) { SourcePostId = ebeUrl.ID });
					}
				}
			}

			foreach (BlogPost blogPost in blogPosts)
			{
				foreach (Match match in localPostRegex.Matches(blogPost.Body))
				{
					// reference to another post within this blog
					blogPost.AddResource(new LocalResource(BlogUrl, match.Groups["url"].Value) { SourcePostId = Convert.ToInt32(match.Groups["id"].Value) });
				}

				foreach (Match match in relativeRegex.Matches(blogPost.Body))
				{
					blogPost.AddResource(new LocalResource(BlogUrl, match.Groups["url"].Value));
				}
				foreach (Match match in absoluteRegex.Matches(blogPost.Body))
				{
					blogPost.AddResource(new LocalResource(BlogUrl, match.Groups["url"].Value));
				}
				if (absoluteHttpRegex != null)
				{
					// if the source blog uses https, also check for http
					foreach (Match match in absoluteHttpRegex.Matches(blogPost.Body))
					{
						blogPost.AddResource(new LocalResource(BlogUrl, match.Groups["url"].Value));
					}
				}
			}

			_log.Info("Found " + blogPosts.Count(p => p.LocalResources.Count > 0) + " posts with references to local resources.");
		}

		public byte[] DownloadFile(string fileUrl)
		{
			byte[] bytes;
			var webRequest = WebRequest.Create(fileUrl);
			using (WebResponse webResponse = webRequest.GetResponse())
			using (Stream responseStream = webResponse.GetResponseStream())
			using (var reader = new StreamReader(responseStream, Encoding.Default))
			{
				string content = reader.ReadToEnd();
				bytes = Encoding.Default.GetBytes(content);
			}
			return bytes;
		}

		/// <summary>
		/// read values from file "replacestrings.txt" in the current directory. Each line contains "string to be replaced;#new string value"
		/// </summary>
		/// <returns></returns>
		private Dictionary<string, string> GetReplaceStrings()
		{
			var dictionary = new Dictionary<string, string>();
			var replace = new FileInfo(AppDomain.CurrentDomain.BaseDirectory + "\\replacestrings.txt");
			if (!replace.Exists)
			{
				return dictionary;
			}

			using (StreamReader streamReader = File.OpenText(replace.FullName))
			{
				var inputString = streamReader.ReadLine();
				while (inputString != null)
				{
					var values = inputString.Split(new[] { ";#" }, StringSplitOptions.RemoveEmptyEntries);
					if (values.Length == 2)
					{
						dictionary.Add(values[0], values[1]);
					}
					inputString = streamReader.ReadLine();
				}
			}

			return dictionary;
		}
	}
}